/*

 fifo.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <errno.h>

#include "fifo.h"
#include "crc8.h"


static FILE *fifoi;
static FILE *fifoo;

static const char *drivername(void) {
    return "FIFO";
}

static int initialize(const char *UNUSED(dev), int UNUSED(unused), FILE *lf) {
    fifoi = fopen("/tmp/usbo", "rb");
    if (!fifoi) {
        fprintf(lf, "Cannot open input fifo\n");
        return -1;
    }
    fifoo =fopen("/tmp/usbi", "r+");
    if (!fifoo) {
        fprintf(lf, "Cannot open output fifo\n");
        return -1;
    }
    return 0;
}

static int getb() {
    int a;
    a=fgetc(fifoi);
    if (a < 0) return -EIO;
    crc_add(a);
    return (unsigned char)a;
}

static int sendb(unsigned char a) {
    crc_add(a);
    fputc(a,fifoo);
    fflush(fifoo);
    return 0;
}

static int getbytes(unsigned short int bytes, unsigned char *data) {
    int i;
    fread(data, bytes, 1, fifoi);
    for (i=0;i < bytes;i++) crc_add(data[i]);
    return 0;
}

static int sendbytes(unsigned short int bytes, unsigned char *data) {
    int i;
    for (i=0;i < bytes;i++) crc_add(data[i]);
    fwrite(data, bytes, 1, fifoo);
    fflush(fifoo);
    return 0;
}

static void shutdown(void) {
    fclose(fifoi);
    fclose(fifoo);
    return;
}

static int reinitialize(FILE *UNUSED(lf)) {
    return 0;
}

static int flush(void) {
    return fflush(fifoo);
}

static void clean(void) {
    return;
}

static int waitb(void) {
    int i;
    i=fgetc(fifoi);
    if (i < 0) return -ENODEV;
    crc_add(i);
    return (unsigned char)i;
}

const struct io_ops_s fifo_io_ops = {
    .drivername   = drivername,
    .initialize   = initialize,
    .reinitialize = reinitialize,
    .getb         = getb,
    .sendb        = sendb,
    .getbytes     = getbytes,
    .sendbytes    = sendbytes,
    .shutdown     = shutdown,
    .flush        = flush,
    .wait         = waitb,
    .clean        = clean,
};
